/****************************************************************************
                             Program QRICHT

                     20 November, 1996     Marc de Lignie
*****************************************************************************
Possible improvements:
- take into account the Earth's curvature
- ignore showers with a radiant below the horizon
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "astro.h"

#define R   6378.         /* Earth's radius */

azh(float,float,float,float,float,float *,float *),
dedm(float,float *,float *),
rahm(float,float *,float *),
JULDATE(int,int,int,float,float,float,float *,float *); /* Meer dan julianday!*/

main(int argc, char *argv[])
{
char str[82],strname[31];
int i,j,nday,nst,day1,mon1,year1,day2,mon2,year2,monb,monm,mone,dayb,daym,daye;
float jdb,jde,jdm;
float sign,phir,labr,phi[4],lab[4],deg,min,sec,
      height,az[4],elv[4],jd,jd1,jd2,gst,t,tt,tb,te,t24,
      ra[4],u[4],de[4],rah[4],ram[4],ded[4],dem[4],
      rast,dest,drast,ddest,rr[3],q;
FILE *fpr,*fpw;
struct meteor met1,met2;

if (argc==1) {
  puts("Usage: qricht filename");
  puts("\nSee qricht.frm for the right input format\n");
  fpw=fopen("qricht.frm","w");
  if (!fpw) {
    printf("\nDisk full ...\n");
    exit(1);
  }
  fputs(" Start date:  1996 10 29\n",fpw);
  fputs(" End date:    1996 11  3\n",fpw);
  fputs(" Height of aiming point (km): 100\n",fpw);
  fputs(" Latitude  of aiming point:   52 12 47 \n",fpw);
  fputs(" Longitude of aiming point:    3 34  7 \n",fpw);
  fputs(" Latitude  of station 1:      53 11 53 \n",fpw);
  fputs(" Longitude of station 1:       2 44 18 \n",fpw);
  fputs(" Latitude  of station 2:      51 18 33 \n",fpw);
  fputs(" Longitude of station 2:       2 24 38 \n",fpw);
  fputs("[Latitude  of station 3:      53 23 43]\n",fpw);
  fputs("[Longitude of station 3:       3 14  6]\n",fpw);
  fputs("[Latitude  of station 4:      53 22 58]\n",fpw);
  fputs("[Longitude of station 4:       4 17 18]\n",fpw);
  exit(0);
}
fpw=fopen("qricht.out","w");
if (!fpw) {
   printf("\nDisk full ...\n");
   exit(1);
}
fpr=fopen(argv[1],"r");
if (!fpr) {
  printf("\nInput file does not exist\n");
  exit(1);
}
fgets(str,80,fpr);
sscanf(str+12,"%i%i%i",&year1,&mon1,&day1);
fgets(str,80,fpr);
sscanf(str+12,"%i%i%i",&year2,&mon2,&day2);
JULDATE(year2,mon2,day2,0.,0.,0.,&jd2,&gst);
JULDATE(year1,mon1,day1,0.,0.,0.,&jd1,&gst);
if (jd1>jd2) {
  printf("Start date is later than end date. Please correct.\n");
  exit(1);
}
fgets(str,80,fpr);
sscanf(str+29,"%f",&height);
fprintf(fpw,"Height of aiming point: %3.0f km\n\n",height);
fgets(str,80,fpr);
sscanf(str+29,"%f%f%f",&deg,&min,&sec);
fprintf(fpw,"Geographical latitude of aiming point:  %2.0f %2.0f\' %2.0f\"\n",
                                                                 deg,min,sec);
if (deg>=0.) sign=+1.;
else         sign=-1.;
phir=PI/180.*(deg+sign*min/60.+sign*sec/3600.);
fgets(str,80,fpr);
sscanf(str+29,"%f%f%f",&deg,&min,&sec);
fprintf(fpw,"Geographical longitude of aiming point: %2.0f %2.0f\' %2.0f\"\n\n",
                                                                 deg,min,sec);
if (deg>=0.) sign=+1.;
else         sign=-1.;
labr=PI/180.*(deg+sign*min/60.+sign*sec/3600.);
fgets(str,80,fpr);
i=0;
do {
  sscanf(str+29,"%f%f%f",&deg,&min,&sec);
  fprintf(fpw,"Geographical latitude of station %i:     %2.0f %2.0f\' %2.0f\"\n",
                                                              i+1,deg,min,sec);
  if (deg>=0.) sign=+1.;
  else         sign=-1.;
  phi[i]=PI/180.*(deg+sign*min/60.+sign*sec/3600.);
  fgets(str,80,fpr);
  sscanf(str+29,"%f%f%f",&deg,&min,&sec);
  fprintf(fpw,"Geographical longitude of station %i:    %2.0f %2.0f\' %2.0f\"\n\n",
                                                              i+1,deg,min,sec);
  if (deg>=0.) sign=+1.;
  else         sign=-1.;
  lab[i]=PI/180.*(deg+sign*min/60.+sign*sec/3600.);
  str[0]=0;
  fgets(str,80,fpr);
  i++;
} while (strlen(str)>29);
nst=i;
fclose(fpr);

/* calculate azimuths and heights for all stations */

for (i=0;i<nst;i++) {
  azh(phir,labr,height,phi[i],lab[i],&az[i],&elv[i]);
  fprintf(fpw,"Azimuth for station %1i:  %7.1f\n",i+1,az[i]/PS);
  fprintf(fpw,"Elevation for station %1i:%7.1f\n\n",i+1,elv[i]/PS);
}

/* calculate equatorial coordinates for given dates */

for (i=0;i<nst;i++) {
  azhude(az[i],elv[i],phi[i],&u[i],&de[i]);
/* in uren, gecorrigeerd voor geografische lengte */
  ra[i]=-u[i]*12/PI+gst+lab[i]/PS/15.;
  de[i]/=PS;
  dedm(de[i],&ded[i],&dem[i]);
}
jd=jd1;
nday=(int)(jd2-jd1+0.1);
tb=floor(17.5-labr*12./PI);         /* take into account time zones */
te=floor( 7.5-labr*12./PI);
if (tb>=24.) tb-=24.;
if (tb<  0.) tb+=24.;
if (te>=24.) te-=24.;
if (te<  0.) te+=24.;
if (te>tb) nday++;
else       te+=24.;
if (nday==0) nday++;
fprintf(fpw,"Julian Day %10.1f corresponds to date %2i-%2i-%4i\n\n",
                                                jd1,mon1,day1,year1);
fprintf(fpw,"    JD     time(UT)    ");
for (j=0;j<nst;j++) fprintf(fpw,"RA%1i    DE%1i     ",j+1,j+1);
for (i=0;i<nday;i++) {
   if (tb<10.1 && tb!=0.) fprintf(fpw,"\n\n%9.1f   0:00:00\n",jd);
   else                   fprintf(fpw,"\n");
   for (t=tb;t<te;t+=1.) {
     if (t>23.9) t24=t-24.;
     else        t24=t;
     if (fabs(t24)>0.1) fprintf(fpw,"\n           %2.0f:00:00",t24);
     else  fprintf(fpw,"\n%9.1f   0:00:00",(tb==0.)?jd:jd+1.);
     for (j=0;j<nst;j++) {
       rahm(ra[j]+t*STF,&rah[j],&ram[j]);
       fprintf(fpw,"%5.0f %02.0f%4.0f %02.0f",rah[j],ram[j],ded[j],dem[j]);
     }
   }
   jd+=1.;
   for (j=0;j<nst;j++) ra[j]+=24.*(STF-1.);
}
/*
  Calculate angles of converence for each stream for the centre date
  of the specified period
*/
for (j=0;j<nst;j++) ra[j]-=nday*12.*(STF-1.);  /* middelste dag */
fpr=fopen("qricht.dat","r");
if (!fpr) {
  printf("\n\nStream info in QRICHT.DAT has disappeared!");
  exit(1);
}
fgets(str,81,fpr);  /* Remove header from input stream */
fgets(str,81,fpr);
fprintf(fpw,"\n\n\nAngles between average trails of active major streams:");
fprintf(fpw,"\n\nStream              time(UT)");
for (j=0;j<nst-1;j++) for (i=j+1;i<nst;i++) fprintf(fpw,"%6i-%1i",j+1,i+1);
do {
  fgets(str,81,fpr);
  sscanf(str,"%18s %i/%i %i/%i %i/%i%f%f%f%f",strname,&monb,&dayb,&mone,
	                  &daye,&monm,&daym,&rast,&dest,&drast,&ddest);
  if (monb>6.) JULDATE(year1,monb,dayb,0.,0.,0.,&jdb,&gst); /* jaarwissel */
  else         JULDATE(year2,monb,dayb,0.,0.,0.,&jdb,&gst);
  if (mone>6.) JULDATE(year1,mone,daye,0.,0.,0.,&jde,&gst);
  else         JULDATE(year2,mone,daye,0.,0.,0.,&jde,&gst);
  if (monm>6.) JULDATE(year1,monm,daym,0.,0.,0.,&jdm,&gst);
  else         JULDATE(year2,monm,daym,0.,0.,0.,&jdm,&gst);
  if ( (jdb>jd1 && jdb<jd2) || (jde>jd1 && jde<jd2) || (jdb<jd1&&jde>jd2) ) {
    rast+=drast*((jd1+jd2)/2.-jdm);   /* radiantdrift */
    dest+=ddest*((jd1+jd2)/2.-jdm);
    rade_r(rast*PS,dest*PS,met1.rb);
    rade_r(rast*PS,dest*PS,met2.rb);
    rade_r(rast*PS,dest*PS,rr);
    fprintf(fpw,"\n\n%-18s ",strname);
    for (t=tb;t<te;t+=1.) {
      if (t<23.9) tt=t;
      else        tt=t-24;
      fprintf(fpw,"\n%22.0f:00:00",tt);
      for (j=0;j<nst-1;j++) for (i=j+1;i<nst;i++) {
        rade_r((ra[j]+tt*STF)*PS*15.,de[j]*PS,met1.re);
        rade_r((ra[i]+tt*STF)*PS*15.,de[i]*PS,met2.re);
        qqq(met1,met2,rr,&q);
        if (fabs(in_p(rr,met1.re))>0.94 || fabs(in_p(rr,met2.re))>0.9) q*=-1.;
        fprintf(fpw,"%8.0f",q/PS);
      }
    }
  }
} while (!feof(fpr));
fprintf(fpw,"\n\n\nA minus sign indicates that the angle varies strongly across the");
fprintf(fpw,"\nfield of either station (close to radiant).");
fcloseall();
return;
}


azh(float phir,float labr,float height,float phi,float lab,
    float *az,float *elv)
{
   float dx,dy,dr;

   dx=labr-lab;
   if (dx>PI) dx-=2.*PI;
   if (dx<-PI) dx+=2.*PI;
   dx=R*dx*cos(phir);
   dy=R*(phir-phi);
   dr=sqrt(dx*dx+dy*dy+height*height);
   if (dx==0. && dy==0.) {
     *az=0.;
     *elv=PI/2.;
   } else {
     *az=atan2(dy,dx);           /* counter-clock with east=0 */
     *az=2.*PI-*az;              /* clockwise with east=0 */
     *az=*az-PI/2.;              /* clockwise with south=0 */
     if (*az<0.) *az+=2.*PI;
     if (*az>2.*PI) *az-=2.*PI;
     *elv=asin(height/dr);
   }
   return;
}


dedm(float x,float *hd,float *min)
{
   float sign;

   if (x>=0.) sign=+1.;
   else       sign=-1.;
   *hd=sign*floor(sign*x);
   *min=(sign*x-*hd*sign)*60.;
   return;
}


rahm(float x,float *hd,float *min)
{
   while (x>=24.) x-=24.;
   while (x<0.00) x+=24.;
   *hd=floor(x);
   *min=(x-*hd)*60.;
   return;
}


/*     THIS SUBROUTINE CALCULATES JULIAN DATE AND GREENWICH
C      SIDERIAL TIME FOR THE INSTANT OF OBSERVATION FROM GIVEN
C      DATE AND TIME (UT)
C      INPUT PARAMETERS : IYR    = YEAR  (INT.)
C                         IMO    = MONTH (INT.)
C                         IDAY   = DAY   (INT.)
C                     TH,TM,TS   = UT (HR,MIN,SEC)
C
C      OUTPUT PARAMETERS   JD    = JULIAN DATE
C                          ST    = GREENWICH SID.TIME (DEC.HR.)
C
C      CALL SUBROUTINE   : CALL JULDATE(IYR,IMO,IDAY,TH,TM,TS,JD,ST)
C
C      REQUIRED SUBROUTINES    : NONE
C

       Source: 	Ondrejov Observatory, Czech Republic                
*/

JULDATE(int IYR,int IMO,int IDAY,float TH,float TM,float TS,
                                                   float *JD,float *ST)
{     long int IJD;
      double H1,H2,H3,H4,T;

      if (2.5>IMO) {
      IYR=IYR-1;
      IMO=IMO+12;
      }
      IJD=IDAY+2+((IYR/100)/4)-(IYR/100);
      IJD=IJD+(long)floor(IYR*3.6525e+2);
      IJD=IJD+(long)floor((IMO+1)*30.6001);
      *JD=(float)(IJD+1.7209945e6);
      H1=((*JD-2.415020e6)/3.6525e4);
      H1=H1*H1;
      H1=(H1*1.075e-6)+.276919398;
      H2=(*JD-2.415020e6)/(3.6525e4)*1.000021359e2;
      H3=H1+H2;
      H4=H3-floor(H3);
      T=TH+(TM/60.)+(TS/3600.);
      *ST=H4*24+(T*1.0027379093);
      return;
}
